#define vmr	vmr_		/* parameter for Name Translation is l_ */
#define ROUTINE int
/* @(#)vecsubs.fc	1.2 (ESO-DMD) 05/20/03 09:41:54 */
/* =======================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
 	Internet e-mail: midas@eso.org
 	Postal address: European Southern Observatory
 			Data Management Division 
 			Karl-Schwarzschild-Strasse 2
 			D 85748 Garching bei Muenchen 
 			GERMANY
 ======================================================================== */
 
 
/* ++++++++++++++++++++++++++++++++++++++++++++++++++
 
.IDENTIFICATION
 holds:		OPFC,	OPFF
   K. Banse                        ESO - Garching
 
.KEYWORDS
   bulk data frames, arithmetic operations
 
.PURPOSE
   Fortran to C interface for the "real" functions 
 
.ALGORITHM
   do it according to opcode ( `, +, -, *, / or =)
   ` means ** (to the power of)
   special condition handler is used for division by "zero"
 
.INPUT/OUTPUT
   call as OPFC(OPCODE,A,SCALR,C,NDIM,USRNUL,NCNT)
           OPFF(OPCODE,A,B,C,NDIM,USRNUL,NCNT)
 
   input par:
   OPCODE:      char.exp.      operation code in postfix notation, e.g. "FF+"
   A:           R*4 array      1. input array
   B:           R*4 array      2. input array
   SCALR:       R*4            scalar value
   NDIM:        I*4            size of arrays involved
   USRNUL:      R*4            user null value
 
   output par:
   C:           R*4 array      result array
   NCNT:	I*4	       null count 

.VERSION
 030514		last modif

 -------------------------------------------------- */
 
#include <midas_def.h>
#include <proto_gen.h>
#include <ftoc.h>
/*

*/

/*
OJO: do not pass single float values, they get passed as double,
     always use pointers 
*/

#if 0     /* ==== Original Code ==== */
SUBROUTINE OPFC(opcode,a,rconst,c,ndim,usrnul,ncount)

CHARACTER   opcode;             /* IN: job to do */
float     *a;
float     *rconst;
float     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define OPFC	opfc_
OPFC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = OpFrameCons(STRIPPED_STRING(1,7,0),PARAM(2,7,1,float *),PARAM(3,7,1,float *),PARAM(4,7,1,float *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE OPFF(opcode,a,b,c,ndim,usrnul,ncount)

CHARACTER   opcode;             /* IN: job to do */
float     *a;
float     *b;
float     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define OPFF	opff_
OPFF(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = OpFrameFrame(STRIPPED_STRING(1,7,0),PARAM(2,7,1,float *),PARAM(3,7,1,float *),PARAM(4,7,1,float *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}
/*

*/

/* ++++++++++++++++++++++++++++++++++++++++++++++++++

.IDENTIFICATION
  subroutine	FUN1F,	FUN2FF,	FUN2FC
  K. Banse                        ESO - Garching

.KEYWORDS
  bulk data frames, functions

.PURPOSE
  apply usual mathematical functions to frames and constants

.ALGORITHM
  all trigonometric functions work with degrees...

.INPUT/OUTPUT
  call as FUN1F(CFUNC,A,C,NDIM,USRNUL,NCNT)
         FUN2FF(CFUNC,A,B,C,NDIM,USRNUL,NCNT)
         FUN2FC(CFUNC,A,SCALR,C,NDIM,USRNUL,NCNT)

  input par:
  CFUNC:     char.exp.      function to do
  A:         R*4 array      1. input array
  B:         R*4 array      2. input array
  NDIM:      I*4            size of arrays involved

  output par:
  C:         R*4 array      result array

.VERSION
 030514		last modif

-------------------------------------------------- */

/*

*/

#if 0     /* ==== Original Code ==== */
SUBROUTINE FUN1F(cfunc,a,c,ndim,usrnul,ncount)

CHARACTER   cfunc;             /* IN: job to do */
float     *a;
float     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define FUN1F	fun1f_
FUN1F(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(6,6,1,fint2c *) = Func1Frame(STRIPPED_STRING(1,6,0),PARAM(2,6,1,float *),PARAM(3,6,1,float *),*PARAM(4,6,1,fint2c *),PARAM(5,6,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE FUN2FF(cfunc,a,b,c,ndim,usrnul,ncount)

CHARACTER   cfunc;             /* IN: job to do */
float     *a;
float     *b;
float     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define FUN2FF	fun2ff_
FUN2FF(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = Func2Frame(STRIPPED_STRING(1,7,0),PARAM(2,7,1,float *),PARAM(3,7,1,float *),PARAM(4,7,1,float *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE FUN2FC(cfunc,a,scalr,c,ndim,usrnul,ncount)

CHARACTER   cfunc;             /* IN: job to do */
float     *a;
float     *scalr;
float     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define FUN2FC	fun2fc_
FUN2FC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = Func2FrameCon(STRIPPED_STRING(1,7,0),PARAM(2,7,1,float *),PARAM(3,7,1,float *),PARAM(4,7,1,float *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}
/*

*/

/* ++++++++++++++++++++++++++++++++++++++++++++++++++

.IDENTIFICATION
  holds:	DOPFC, 	DOPFF
  K. Banse                        ESO - Garching

.KEYWORDS
  double precision, bulk data frames, arithmetic operations

.PURPOSE
  perform arithmetic on arrays

.ALGORITHM
  do it according to opcode ( `, +, -, *, / or =)
  ` means ** (to the power of)
  special condition handler is used for division by "zero"

.INPUT/OUTPUT
  call as DOPFC(OPCODE,A,SCALR,C,NDIM,USRNUL,NCNT)
          DOPFF(OPCODE,A,B,C,NDIM,USRNUL,NCNT)

  input par:
  OPCODE:      char.exp.      operation code in postfix notation, e.g. "FF+"
  A:           R*8 array      1. input array
  B:           R*8 array      2. input array
  SCALR:       R*8            scalar value
  NDIM:        I*4            size of arrays involved

  output par:
  C:           R*8 array      result array

.VERSION
 030515		last modif

--------------------------------------------------  */

#if 0     /* ==== Original Code ==== */
SUBROUTINE DOPFC(opcode,a,scalr,c,ndim,usrnul,ncount)

CHARACTER   opcode;             /* IN: job to do */
double     *a;
double     *scalr;
double     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define DOPFC	dopfc_
DOPFC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = DOpFrameCons(STRIPPED_STRING(1,7,0),PARAM(2,7,1,double *),PARAM(3,7,1,double *),PARAM(4,7,1,double *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE DOPFF(opcode,a,b,c,ndim,usrnul,ncount)

CHARACTER   opcode;             /* IN: job to do */
double     *a;
double     *b;
double     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define DOPFF	dopff_
DOPFF(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = DOpFrameFrame(STRIPPED_STRING(1,7,0),PARAM(2,7,1,double *),PARAM(3,7,1,double *),PARAM(4,7,1,double *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}
/*

*/
 
/* ++++++++++++++++++++++++++++++++++++++++++++++++

.IDENTIFICATION
 holds:		DFN1F, DFN2FF, DFN2FC
 K. Banse                        ESO - Garching

.KEYWORDS
  bulk data frames, functions

.PURPOSE
  apply usual mathematical functions to frames and constants

.ALGORITHM
  use special condition handler NULLER to trap invalid input to functions
  all trigonometric functions work with degrees...

.INPUT/OUTPUT
  call as DFN1F(CFUNC,A,C,NDIM,USRNUL,NCNT)
          DFN2FF(CFUNC,A,B,C,NDIM,USRNUL,NCNT)
          DFN2FC(CFUNC,A,SCALR,C,NDIM,USRNUL,NCNT)

  input par:
  CFUNC:      char.exp.      input function
  NDIM:       I*4            size of arrays involved
  A:          R*8 array      1. input array
  B:          R*8 array      2. input array
  SCALR:       R*8            input scalar value

  output par:
  C:          R*8 array      result array

.VERSION
 030515		last modif

-------------------------------------------------- */

#if 0     /* ==== Original Code ==== */
SUBROUTINE DFN1F(cfunc,a,c,ndim,usrnul,ncount)

CHARACTER   cfunc;             /* IN: job to do */
double     *a;
double     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define DFN1F	dfn1f_
DFN1F(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(6,6,1,fint2c *) = DFunc1Frame(STRIPPED_STRING(1,6,0),PARAM(2,6,1,double *),PARAM(3,6,1,double *),*PARAM(4,6,1,fint2c *),PARAM(5,6,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE DFN2FF(cfunc,a,b,c,ndim,usrnul,ncount)

CHARACTER   cfunc;             /* IN: job to do */
double     *a;
double     *b;
double     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define DFN2FF	dfn2ff_
DFN2FF(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = DFunc2Frame(STRIPPED_STRING(1,7,0),PARAM(2,7,1,double *),PARAM(3,7,1,double *),PARAM(4,7,1,double *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

#if 0     /* ==== Original Code ==== */
SUBROUTINE DFN2FC(cfunc,a,scalr,c,ndim,usrnul,ncount)

CHARACTER   cfunc;             /* IN: job to do */
double     *a;
double     *scalr;
double     *c;
fint2c    *ndim;
float     *usrnul;
fint2c    *ncount;
#else     /* ==== Generated Code === */
#define DFN2FC	dfn2fc_
DFN2FC(va_alist) va_dcl 
{ va_list Cargs;
#endif    /* ======================= */
  int FORmark;           /* <forif> */
  va_start(Cargs);       /* <forif> */
  FORmark = ftoc_mark(); /* <forif> */

*PARAM(7,7,1,fint2c *) = DFunc2FrameCon(STRIPPED_STRING(1,7,0),PARAM(2,7,1,double *),PARAM(3,7,1,double *),PARAM(4,7,1,double *),*PARAM(5,7,1,fint2c *),PARAM(6,7,1,float *));
  ftoc_free(FORmark);    /* <forif> */
}

